package loci.formats;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;

import org.apache.commons.vfs2.FileObject;
import org.apache.commons.vfs2.FileSystemManager;
import org.apache.commons.vfs2.VFS;

public class HiddenTest {

  private String file;
  private long repeatCount;

  public HiddenTest(String file, long repeatCount) {
    this.file = file;
    this.repeatCount = repeatCount;
  }

  public void run() {
    testFileHidden();
    testFilesHidden();
    testDOSHidden();
    testFileInputStream();
    testVFSHidden();
  }

  private void testFileHidden() {
    long t0 = System.currentTimeMillis();
    for (long i=0; i<repeatCount; i++) {
      File f = new File(file);
      f.isHidden();
    }
    long t1 = System.currentTimeMillis();
    logTime(t1 - t0, "java.io.File");
  }

  private void testFilesHidden() {
    long t0 = System.currentTimeMillis();
    for (long i=0; i<repeatCount; i++) {
      Path p = Paths.get(file);
      try {
        Files.isHidden(p);
      }
      catch (IOException e) { }
    }
    long t1 = System.currentTimeMillis();
    logTime(t1 - t0, "java.nio.file.Files");
  }

  private void testDOSHidden() {
    long t0 = System.currentTimeMillis();
    for (long i=0; i<repeatCount; i++) {
      Path p = Paths.get(file);
      try {
        Files.getAttribute(p, "dos:hidden");
      }
      catch (IOException e) { }
    }
    long t1 = System.currentTimeMillis();
    logTime(t1 - t0, "dos:hidden");
  }

  private void testFileInputStream() {
    long t0 = System.currentTimeMillis();
    for (long i=0; i<repeatCount; i++) {
      try {
        FileInputStream fis = new FileInputStream(file);
        fis.read();
        fis.close();
      }
      catch (IOException e) { }
    }
    long t1 = System.currentTimeMillis();
    logTime(t1 - t0, "FileInputStream");
  }

  private void testVFSHidden() {
    long t0 = System.currentTimeMillis();
    try {
      FileSystemManager fsManager = VFS.getManager();
      for (long i=0; i<repeatCount; i++) {
        FileObject fo = fsManager.resolveFile(file);
        fo.isHidden();
      }
    }
    catch (Exception e) {
      e.printStackTrace();
    }
    long t1 = System.currentTimeMillis();
    logTime(t1 - t0, "Apache Commons VFS");
  }

  private void logTime(long totalTime, String method) {
    System.out.println(method + " total time (" + repeatCount + ")  calls: " + totalTime + " ms");
    System.out.println(method + " avg time per call: " + (totalTime / (double) repeatCount) + " ms");
  }

  public static void main(String[] args) throws Exception {
    HiddenTest test = new HiddenTest(args[0], Long.parseLong(args[1]));
    test.run();
  }

}
